# Do depth-first recursive directory scannig
#
# Usage:
#
#   DirScan.preorder if want_to_scan_in_preorder
#   DirScan.postorder if want_to_scan_in_postorder
#   DirScan.scan "file1", "file2", ... do
#     |f|
#     prune if file_f_is_not_desired_to_be_processed
#     some_work_using_file_name f
#   end

module DirScan
  Config = { :preorder => true }

  def preorder
    Config[:preorder] = true
  end

  def postorder
    Config[:preorder] = false
  end

  def scan(*dirs)
    proc = Proc.new
    do_scan(dirs, proc)
  end

  def do_scan(dirs, proc)
    while file = dirs.shift
      catch(:prune) do
	proc.call file if Config[:preorder]
	if File.directory? file
	  Dir.open(file) do
	    |dir|
	    for f in dir
	      next if f =~ /^\.\.?$/
	      if file == "/"
		f = "/"+f
	      else
		f = file+"/"+f
	      end
	      do_scan [f], proc
	    end
	  end
	end
	proc.call file unless Config[:preorder]
      end
    end
  end

  def prune
    throw :prune
  end

  module_function :preorder, :postorder, :scan, :do_scan, :prune
end

if $0 == __FILE__
  puts "preorder depth-first scanning"
  DirScan.preorder
  DirScan.scan(".") { |f| puts f }

  puts "postorder depth-first scanning"
  DirScan.postorder
  DirScan.scan(".") { |f| puts f }
end
